package autonat_test

import (
	"context"
	"testing"
	"time"

	"gitlab.dms3.io/p2p/go-p2p"
	autonat "gitlab.dms3.io/p2p/go-p2p-autonat"
	"gitlab.dms3.io/p2p/go-p2p-core/event"
	"gitlab.dms3.io/p2p/go-p2p-core/network"
)

// This separate testing package helps to resolve a circular dependency potentially
// being created between p2p and p2p-autonat

func TestAutonatRoundtrip(t *testing.T) {
	ctx, cancel := context.WithCancel(context.Background())
	defer cancel()

	// 3 hosts are used: [client] and [service + dialback dialer]
	client, err := p2p.New(ctx, p2p.ListenAddrStrings("/ip4/127.0.0.1/tcp/0"), p2p.EnableAutoNAT())
	if err != nil {
		t.Fatal(err)
	}

	service, err := p2p.New(ctx, p2p.ListenAddrStrings("/ip4/127.0.0.1/tcp/0"))
	if err != nil {
		t.Fatal(err)
	}
	dialback, err := p2p.New(ctx, p2p.NoListenAddrs)
	if err != nil {
		t.Fatal(err)
	}
	_, err = autonat.New(ctx, service, autonat.EnableService(dialback.Network(), true))
	if err != nil {
		t.Fatal(err)
	}

	client.Peerstore().AddAddrs(service.ID(), service.Addrs(), time.Hour)
	err = client.Connect(ctx, service.Peerstore().PeerInfo(service.ID()))
	if err != nil {
		t.Fatal(err)
	}

	cSub, err := client.EventBus().Subscribe(new(event.EvtLocalReachabilityChanged))
	if err != nil {
		t.Fatal(err)
	}
	defer cSub.Close()

	select {
	case stat := <-cSub.Out():
		if stat == network.ReachabilityUnknown {
			t.Fatalf("After status update, client did not know its status")
		}
	case <-time.After(30 * time.Second):
		t.Fatal("sub timed out.")
	}
}
